#include <opencv2/opencv.hpp>
#include "NativeLogging.h"

using namespace std;
using namespace cv;

static const char* TAG = "HDR";

// Part 1: Load Images.
// Read the images at the locations in inputImagePaths
// and store them in the images vector.
// Helpful OpenCV functions: imread.
static void read_input_images(const vector<string>& inputImagePaths, vector<Mat>& images)
{

}

// Part 2: Image Registration.
// Ideally, you expect the images to be perfectly aligned.
// However, as this may not necessarily be the case, you'll process
// the provided images so that they're all aligned.
// Replace the images in the provided vector with the aligned versions.
//
// Tip: The first preset image pair is pre-aligned, so you can initially
// skip this stage to verify that the rest of your pipeline is correct.
//
// Helpful OpenCV classes and functions:
// ORB, BFMatcher, findHomography, warpPerspective
// The above classes and functions are just recommendations.
// You are free to experiment with other methods as well.
//
// Hint: The outline for this will be quite similar to
// the find_correspondences function in Panorama.cpp.
static void align_images(vector<Mat>& images)
{

}

// Part 3: Weight Calculation.
// Compute the weights for each image and place them in the given weights vector.
// Hint: cv::Laplacian, cv::split, cv::merge.
static void compute_weights(const vector<Mat>& images, vector<Mat>& weights)
{

}

// Part 4: Laplacian Pyramid Blending.
// Blend the images using the provided weights and save the result in output.
// Your pyramid should have (maxPyrIndex+1) levels. Note that many pyramid
// related OpenCV functions accept the max index as the argument,
// rather than the number of levels. Make sure you're not off-by-one.
// Also, make sure that the output is a 3-channel 8-bit per-pixel image.
// Hint: cv::buildPyramid, cv::pyrUp
static void blend_pyramids(const vector<Mat>& images, const vector<Mat>& weights, int maxPyrIndex, Mat& output)
{

}

// This is the main entry point for the exposure-fusion module.
// You do not need to change anything in this function.
// However, you should read through it to understand everything works.
// It accepts two arguments:
// 1. inputImagePaths: A vector of absolute paths of the images to fuse.
// 2. outputImagePath: The path where the output image should be saved.
bool exposure_fusion(const vector<string>& inputImagePaths, const string& outputImagePath)
{
    //Read in the given images.
    vector<Mat> images;
    read_input_images(inputImagePaths, images);

    //Verify that the images were correctly read.
    if(images.size()!=inputImagePaths.size())
    {
        LOG_ERROR(TAG, "Images were not read in correctly!");
        return false;
    }

    //Verify that the images are RGB images of the same size.
    const int numChannels = 3;
    Size imgSize = images[0].size();
    for(const Mat& img: images)
    {
        if(img.channels()!=numChannels)
        {
            LOG_ERROR(TAG, "CreateHDR expects 3 channel RGB images!");
            return false;
        }
        if(img.size()!=imgSize)
        {
            LOG_ERROR(TAG, "HDR images must be of equal sizes!");
            return false;
        }
    }
    LOG_DEBUG(TAG, "%d images successfully read.", images.size());

    //Now we'll make sure that the images line up correctly.
    align_images(images);
    LOG_DEBUG(TAG, "Image alignment complete.");

    //Compute the weights for each image.
    vector<Mat> weights;
    compute_weights(images, weights);
    if(weights.size()!=images.size())
    {
        LOG_ERROR(TAG, "Image weights were not generated!");
        return false;
    }
    LOG_DEBUG(TAG, "Weight computation complete.");

    //Fusion using Laplacian pyramid blending.
    int maxPyrIdx = 6;
    Mat outputImage;
    blend_pyramids(images, weights, maxPyrIdx, outputImage);
    if(outputImage.empty())
    {
        LOG_ERROR(TAG, "Blending did not produce an output!");
        return false;
    }
    LOG_DEBUG(TAG, "Blending complete!");

    //Save output.
    bool result = imwrite(outputImagePath, outputImage);
    if(!result)
    {
        LOG_ERROR(TAG, "Failed to save output image to file!");
    }
    return result;
}
