#include <opencv2/opencv.hpp>
#include "NativeLogging.h"

using namespace std;
using namespace cv;

static const char* TAG = "Panorama";

// Part 1: Load Images.
// Read the images at the locations in inputImagePaths
// and store them in the images vector.
// Helpful OpenCV functions: imread.
static void read_input_images(const vector<string>& inputImagePaths, vector<Mat>& images)
{

}

// Part 2: Find Correspondences.
// This function finds feature correspondences for a given set of images.
// On return, the matches vector should contain a vector of corresponding points for
// each image in the images vector.
// That is, matches[i][k] and matches[j][k] should be the k-th corresponding point
// in images[i] and images[j] respectively.
static void find_correspondences(const vector<Mat>& images, vector<vector<Point2f>>& matches)
{
    //Step 1: Extract features for each image.
    //Hint  : cv::ORB
    
    //Step 2: Match the extracted features.
    //Hint  : cv::BFMatcher,
    
    //Step 3: Prune the matches and populate the matches vector.
    //Hint  : Feature distance ratio test, cv::DMatch members.
}

// Part 3: Compute Homography.
// Use the provided correspondences (matches) to compute a homography
// that transforms points in the right image to points in the left image.
// Make sure your computation is robust to outliers!
// Hint: cv::findHomography.
static void compute_homography(const vector<vector<Point2f>>& matches, Matx33f& H)
{

}

// Part 4: Warp the image.
// Use the provided homography to warp the right image.
// You also need to pad/extend the left image to have the same dimension as
// the warped right image.
// Place the results in outputImages.
// Hint: perspectiveTransform, warpPerspective.
static void warp(const vector<Mat>& inputImages, const Matx33f& H, vector<Mat>& outputImages)
{
    //Step 1: Figure out the dimensions of the warped right image.
    //Hint  : cv::perspectiveTransform.
    
    //Step 2: Warp the right image.
    //Hint  : cv::warpPerspective.
    
    //Step 3: Pad/Extend the left image so that it matches the dimensions of the right image.
    //Hint  : Look into ROIs (region of interest).
}

// Part 5: Blending.
// Implement Laplacian blending to stitch together the warped image stack.
// Make sure that the output is a 3-channel 8-bit per-pixel image!
// You can re-use most of the code from the exposure fusion section.
// Hint: buildPyramid, pyrUp.
static void blend(const vector<Mat>& inputImages, Mat& outputImage)
{

}

// This is the main entry point for the panorama stitching method.
// You do not need to change anything in this function.
// However, you should read through it to understand everything works.
// It accepts two arguments:
// 1. inputImagePaths: A vector of absolute paths of the images to stitch together.
// 2. outputImagePath: The path where the output image should be saved.
bool create_panorama(const vector<string>& inputImagePaths, const string& outputImagePath)
{
    //Load the images.
    vector<Mat> images;
    read_input_images(inputImagePaths, images);
    if(images.size()!=inputImagePaths.size())
    {
        LOG_ERROR(TAG, "Images were not read in correctly!");
        return false;
    }
    if(images.front().empty())
    {
        LOG_ERROR(TAG, "Invalid input image detected!");
        return false;
    }
    LOG_DEBUG(TAG, "%d images successfully read.", images.size());
    
    //Find correspondences.
    vector<vector<Point2f>> matches;
    find_correspondences(images, matches);
    if(matches.size()!=images.size())
    {
        LOG_ERROR(TAG, "Correspondences not set correctly!");
        return false;
    }
    if(matches.front().empty())
    {
        LOG_ERROR(TAG, "No correspondences found!");
        return false;
    }
    LOG_DEBUG(TAG, "Found %d correspondences.", matches.front().size());
    
    //Compute homography.
    const float kDetHomographyThresh = 0.01;
    Matx33f H = Matx33f::zeros();
    compute_homography(matches, H);
    if(determinant(H)<kDetHomographyThresh)
    {
        LOG_ERROR(TAG, "Homography not computed correctly!");
        return false;
    }
    LOG_DEBUG(TAG, "Homography computed.");
    
    //Warp the image.
    vector<Mat> warpedImages;
    warp(images, H, warpedImages);
    if(warpedImages.size()!=images.size())
    {
        LOG_ERROR(TAG, "Warped images not set correctly!");
        return false;
    }
    if(warpedImages.front().empty())
    {
        LOG_ERROR(TAG, "Invalid warped image detected!");
        return false;
    }
    for(auto& img: warpedImages)
    {
        if(img.size()!=warpedImages.front().size())
        {
            LOG_ERROR(TAG, "Warped images must have the same dimension!");
            return false;
        }
    }
    LOG_DEBUG(TAG, "Warping complete.");
    
    //Blend.
    Mat panorama;
    blend(warpedImages, panorama);
    LOG_DEBUG(TAG, "Blending complete!");
    
    //Save the image.
    imwrite(outputImagePath, panorama);
    
    return true;
}
